/* glpios.h (integer optimization suite) */

/***********************************************************************
*  This code is part of GLPK (GNU Linear Programming Kit).
*
*  Copyright (C) 2000, 01, 02, 03, 04, 05, 06, 07 Andrew Makhorin,
*  Department for Applied Informatics, Moscow Aviation Institute,
*  Moscow, Russia. All rights reserved. E-mail: <mao@mai2.rcnet.ru>.
*
*  GLPK is free software: you can redistribute it and/or modify it
*  under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  GLPK is distributed in the hope that it will be useful, but WITHOUT
*  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
*  License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with GLPK. If not, see <http://www.gnu.org/licenses/>.
***********************************************************************/

#ifndef _GLPIOS_H
#define _GLPIOS_H

#define _GLP_TREE
typedef struct glp_tree glp_tree;

#include "glpapi.h"

typedef struct IOSLOT IOSLOT;
typedef struct IOSNPD IOSNPD;
typedef struct IOSBND IOSBND;
typedef struct IOSTAT IOSTAT;
typedef struct IOSROW IOSROW;
typedef struct IOSAIJ IOSAIJ;

struct glp_tree
{     /* branch-and-bound tree */
      DMP *pool;
      /* memory pool to store IOS components */
      /*--------------------------------------------------------------*/
      /* original problem components */
      /* these components correspond to the original MIP and its LP
         relaxation */
      int orig_m;
      /* number of rows */
      int orig_n;
      /* number of columns */
      int *orig_type; /* int orig_type[1+orig_m+orig_n]; */
      /* types of all variables */
      double *orig_lb; /* double orig_lb[1+orig_m+orig_n]; */
      /* lower bounds of all variables */
      double *orig_ub; /* double orig_ub[1+orig_m+orig_n]; */
      /* upper bounds of all variables */
      int *orig_stat; /* int orig_stat[1+orig_m+orig_n]; */
      /* statuses of all variables */
      double *orig_prim; /* double orig_prim[1+orig_m+orig_n]; */
      /* primal values of all variables */
      double *orig_dual; /* double orig_dual[1+orig_m+orig_n]; */
      /* dual values of all variables */
      double orig_obj;
      /* optimal objective value for LP relaxation */
      /*--------------------------------------------------------------*/
      /* branch-and-bound tree */
      int nslots;
      /* length of the array of slots (enlarged automatically) */
      int avail;
      /* index of the first free slot; 0 means all slots are in use */
      IOSLOT *slot; /* MIPSLOT slot[1+nslots]; */
      /* array of slots:
         slot[0] is never used;
         slot[p], 1 <= p <= nslots, either contains a pointer to some
         node of the branch-and-bound tree, in which case p is used on
         API level as the reference number of corresponding subproblem,
         or is free; all free slots are linked into single linked list;
         slot[1] always contains a pointer to the root node (it is free
         only if the tree is empty) */
      IOSNPD *head;
      /* pointer to the head of the active list */
      IOSNPD *tail;
      /* pointer to the tail of the active list */
      /* the active list is a doubly linked list of active subproblems
         which correspond to leaves of the tree; all subproblems in the
         active list are ordered chronologically (each a new subproblem
         is always added to the tail of the list) */
      int a_cnt;
      /* current number of active nodes (including the current one) */
      int n_cnt;
      /* current number of all (active and inactive) nodes */
      int t_cnt;
      /* total number of nodes including those which have been already
         removed from the tree; this count is increased whenever a new
         node is created and never decreased */
      /*--------------------------------------------------------------*/
      /* current subproblem and its LP relaxation */
      IOSNPD *curr;
      /* pointer to the current subproblem (which can be only active);
         NULL means the current subproblem does not exist */
      glp_prob *mip;
      /* modified original MIP problem object; its LP part corresponds
         to LP relaxation of the current subproblem */
      int *non_int; /* int non_int[1+mip->n]; */
      /* these column flags are set once LP relaxation of the current
         subproblem has been solved;
         non_int[0] is not used;
         non_int[j], 1 <= j <= mip->n, is j-th column flag; if this
         flag is set, corresponding structural variable is required to
         be integer, but its value in basic solution is fractional */
      /*--------------------------------------------------------------*/
      /* parent (predecessor) subproblem components */
      /* these components are used to build patch lists on freezing the
         current subproblem */
      int pred_m;
      /* number of rows */
      int pred_n;
      /* number of columns */
      int pred_max;
      /* length of the following four arrays (enlarged automatically),
         pred_max >= pred_m + pred_n */
      int *pred_type; /* int pred_type[1+pred_m+pred_n]; */
      /* types of all variables */
      double *pred_lb; /* double pred_lb[1+pred_m+pred_n]; */
      /* lower bounds of all variables */
      double *pred_ub; /* double pred_ub[1+pred_m+pred_n]; */
      /* upper bounds of all variables */
      int *pred_stat; /* int pred_stat[1+pred_m+pred_n]; */
      /* statuses of all variables */
      /*--------------------------------------------------------------*/
      /* control parameters and statistics */
      int msg_lev;
      /* message level:
         GLP_MSG_OFF - no output
         GLP_MSG_ERR - warning and error messages only
         GLP_MSG_ON  - normal output
         GLP_MSG_ALL - full output
         GLP_MSG_DBG - debug output */
      int br_tech;
      /* branching technique:
         GLP_BR_FFV  - first fractional variable
         GLP_BR_LFV  - last fractional variable
         GLP_BR_MFV  - most fractional variable
         GLP_BR_DT   - heuristic by Driebeck and Tomlin */
      int bt_tech;
      /* backtracking technique:
         GLP_BT_DFS  - depth first search
         GLP_BT_BFS  - breadth first search
         GLP_BT_BLB  - best local bound
         GLP_BT_BPH  - best projection heuristic */
      double tol_int;
      /* absolute tolerance used to check if the current basic solution
         is integer feasible */
      double tol_obj;
      /* relative tolerance used to check if the value of the objective
         function is better than the incumbent objective value */
      int tm_lim;
      /* searching time limit, in milliseconds */
      int out_frq;
      /* output frequency, in milliseconds; this parameter specifies
         how frequently the solver sends information about the progress
         of the search to the terminal */
      int out_dly;
      /* output delay, in milliseconds; this parameter specifies how
         long the LP solver delays output to the terminal on solving LP
         relaxation of the current subproblem; zero means no delay */
      glp_ulong tm_beg;
      /* starting time of the search, in seconds; the total time of the
         search is the difference between utime() and tm_beg */
      glp_ulong tm_lag;
      /* the most recent time, in seconds, at which the progress of the
         the search was displayed */
      /*--------------------------------------------------------------*/
      /* advanced solver interface */
      void (*cb_func)(glp_tree *tree, void *info);
      /* entry point to the event-driven user-defined callback routine;
         NULL means the advanced solver interface is not used */
      void *cb_info;
      /* transit pointer passed to the user-defined callback routine */
      int reason;
      /* flag indicating the reason why the callback routine is being
         called (see glpk.h) */
      int terminate;
      /* flag indicating that the callback routine requires premature
         termination of the search */
};

struct IOSLOT
{     /* node subproblem slot */
      IOSNPD *node;
      /* pointer to subproblem descriptor; NULL means free slot */
      int next;
      /* index of another free slot (only if this slot is free) */
};

struct IOSNPD
{     /* node subproblem descriptor */
      int p;
      /* subproblem reference number (it is the index to corresponding
         slot, i.e. slot[p] points to this descriptor) */
      IOSNPD *up;
      /* pointer to the parent subproblem; NULL means this node is the
         root of the tree, in which case p = 1 */
      int level;
      /* node level (the root node has level 0) */
      int count;
      /* if count = 0, this subproblem is active; if count > 0, this
         subproblem is inactive, in which case count is the number of
         its child subproblems */
      IOSBND *b_ptr;
      /* linked list of rows and columns of the parent subproblem whose
         types and bounds were changed;
         this list is destroyed on reviving and built anew on freezing
         the subproblem */
      IOSTAT *s_ptr;
      /* linked list of rows and columns of the parent subproblem whose
         statuses were changed;
         this list is destroyed on reviving and built anew on freezing
         the subproblem */
      IOSROW *r_ptr;
      /* linked list of rows (cuts) added to the parent subproblem;
         this list is destroyed on reviving and built anew on freezing
         the subproblem */
      double bound;
      /* local lower (minimization) or upper (maximization) bound for
         integer optimal solution to *this* subproblem; this bound is
         local in the sense that only subproblems in the subtree rooted
         at this node cannot have better integer feasible solutions;
         on creating a subproblem its local bound is inherited from its
         parent and then can be made stronger (never weaker); for the
         root subproblem its local bound is initially set to -DBL_MAX
         (minimization) or +DBL_MAX (maximization) and then improved as
         the root LP relaxation has been solved */
      /* if this subproblem is inactive, the following two quantities
         correspond to final optimal solution of its LP relaxation; for
         active subproblems these quantities are undefined */
      int ii_cnt;
      /* number of columns (structural variables) of integer kind whose
         primal values are fractional */
      double ii_sum;
      /* the sum of integer infeasibilities */
      IOSNPD *temp;
      /* auxiliary pointer used by some routines */
      IOSNPD *prev;
      /* pointer to previous subproblem in the active list */
      IOSNPD *next;
      /* pointer to next subproblem in the active list */
};

struct IOSBND
{     /* bounds change entry */
      int k;
      /* ordinal number of corresponding row (1 <= k <= m) or column
         (m+1 <= k <= m+n), where m and n are the number of rows and
         columns, resp., in the parent subproblem */
      int type;
      /* new type */
      double lb;
      /* new lower bound */
      double ub;
      /* new upper bound */
      IOSBND *next;
      /* pointer to next entry for the same subproblem */
};

struct IOSTAT
{     /* status change entry */
      int k;
      /* ordinal number of corresponding row (1 <= k <= m) or column
         (m+1 <= k <= m+n), where m and n are the number of rows and
         columns, resp., in the parent subproblem */
      int stat;
      /* new status */
      IOSTAT *next;
      /* pointer to next entry for the same subproblem */
};

struct IOSROW
{     /* row (cut) addition entry */
      SCS *name;
      /* row name or NULL */
      int type;
      /* row type */
      double lb;
      /* row lower bound */
      double ub;
      /* row upper bound */
      IOSAIJ *ptr;
      /* pointer to the coefficient list */
      double rii;
      /* row scale factor */
      int stat;
      /* row status */
      IOSROW *next;
      /* pointer to next entry for the same subproblem */
};

struct IOSAIJ
{     /* constraint coefficient */
      int j;
      /* column number */
      double val;
      /* coefficient value */
      IOSAIJ *next;
      /* pointer to next coefficient for the same row */
};

#define ios_create_tree _glp_ios_create_tree
glp_tree *ios_create_tree(glp_prob *mip);
/* create branch-and-bound tree */

#define ios_revive_node _glp_ios_revive_node
void ios_revive_node(glp_tree *tree, int p);
/* revive specified subproblem */

#define ios_freeze_node _glp_ios_freeze_node
void ios_freeze_node(glp_tree *tree);
/* freeze current subproblem */

#define ios_clone_node _glp_ios_clone_node
void ios_clone_node(glp_tree *tree, int p, int nnn, int ref[]);
/* clone specified subproblem */

#define ios_delete_node _glp_ios_delete_node
void ios_delete_node(glp_tree *tree, int p);
/* delete specified subproblem */

#define ios_delete_tree _glp_ios_delete_tree
void ios_delete_tree(glp_tree *tree);
/* delete branch-and-bound tree */

#define ios_best_node _glp_ios_best_node
int ios_best_node(glp_tree *tree);
/* find active node with best local bound */

#define ios_relative_gap _glp_ios_relative_gap
double ios_relative_gap(glp_tree *tree);
/* compute relative mip gap */

#define ios_solve_node _glp_ios_solve_node
int ios_solve_node(glp_tree *tree);
/* solve LP relaxation of current subproblem */

#define ios_driver _glp_ios_driver
int ios_driver(glp_tree *tree);
/* branch-and-bound driver */

#endif

/* eof */
