/* glpios01.c */

/***********************************************************************
*  This code is part of GLPK (GNU Linear Programming Kit).
*
*  Copyright (C) 2000, 01, 02, 03, 04, 05, 06, 07 Andrew Makhorin,
*  Department for Applied Informatics, Moscow Aviation Institute,
*  Moscow, Russia. All rights reserved. E-mail: <mao@mai2.rcnet.ru>.
*
*  GLPK is free software: you can redistribute it and/or modify it
*  under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  GLPK is distributed in the hope that it will be useful, but WITHOUT
*  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
*  License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with GLPK. If not, see <http://www.gnu.org/licenses/>.
***********************************************************************/

#include "glpios.h"

/***********************************************************************
*  NAME
*
*  ios_create_tree - create branch-and-bound tree
*
*  SYNOPSIS
*
*  #include "glpios.h"
*  glp_tree *ios_create_tree(glp_prob *mip);
*
*  DESCRIPTION
*
*  The routine ios_create_tree creates the branch-and-bound tree.
*
*  Being created the tree consists of the only root subproblem whose
*  reference number is 1. Note that initially the root subproblem is in
*  frozen state and therefore needs to be revived.
*
*  RETURNS
*
*  The routine returns a pointer to the tree created. */

glp_tree *ios_create_tree(glp_prob *mip)
{     int m = mip->m;
      int n = mip->n;
      glp_tree *tree;
      IOSNPD *node;
      int i, j, p;
      tree = xmalloc(sizeof(glp_tree));
      tree->pool = dmp_create_pool();
      /* save original problem components */
      tree->orig_m = m;
      tree->orig_n = n;
      tree->orig_type = xcalloc(1+m+n, sizeof(int));
      tree->orig_lb = xcalloc(1+m+n, sizeof(double));
      tree->orig_ub = xcalloc(1+m+n, sizeof(double));
      tree->orig_stat = xcalloc(1+m+n, sizeof(int));
      tree->orig_prim = xcalloc(1+m+n, sizeof(double));
      tree->orig_dual = xcalloc(1+m+n, sizeof(double));
      for (i = 1; i <= m; i++)
      {  GLPROW *row = mip->row[i];
         tree->orig_type[i] = row->type;
         tree->orig_lb[i] = row->lb;
         tree->orig_ub[i] = row->ub;
         tree->orig_stat[i] = row->stat;
         tree->orig_prim[i] = row->prim;
         tree->orig_dual[i] = row->dual;
      }
      for (j = 1; j <= n; j++)
      {  GLPCOL *col = mip->col[j];
         tree->orig_type[m+j] = col->type;
         tree->orig_lb[m+j] = col->lb;
         tree->orig_ub[m+j] = col->ub;
         tree->orig_stat[m+j] = col->stat;
         tree->orig_prim[m+j] = col->prim;
         tree->orig_dual[m+j] = col->dual;
      }
      tree->orig_obj = mip->obj_val;
      /* allocate the branch-and-bound tree */
      tree->nslots = 20;
      tree->avail = 0;
      tree->slot = xcalloc(1+tree->nslots, sizeof(IOSLOT));
      tree->head = tree->tail = NULL;
      tree->a_cnt = tree->n_cnt = tree->t_cnt = 0;
      /* initialize the stack of free slots */
      for (p = tree->nslots; p >= 1; p--)
      {  tree->slot[p].node = NULL;
         tree->slot[p].next = tree->avail;
         tree->avail = p;
      }
      /* pull a free slot for the root node */
      p = tree->avail;
      xassert(p == 1);
      tree->avail = tree->slot[p].next;
      xassert(tree->slot[p].node == NULL);
      tree->slot[p].next = 0;
      /* create and initialize the root node subproblem */
      node = dmp_get_atom(tree->pool, sizeof(IOSNPD));
      tree->slot[p].node = node;
      node->p = p;
      node->up = NULL;
      node->level = 0;
      node->count = 0;
      node->b_ptr = NULL;
      node->s_ptr = NULL;
      node->r_ptr = NULL;
      switch (mip->dir)
      {  case GLP_MIN:
            node->bound = -DBL_MAX; break;
         case GLP_MAX:
            node->bound = +DBL_MAX; break;
         default:
            xassert(mip != mip);
      }
      node->ii_cnt = 0;
      node->ii_sum = 0.0;
      node->temp = node->prev = node->next = NULL;
      /* add the root subproblem to the active list */
      tree->head = tree->tail = node;
      tree->a_cnt++;
      tree->n_cnt++;
      tree->t_cnt++;
      /* current subproblem does not exist yet */
      tree->curr = NULL;
      tree->mip = mip;
      tree->non_int = xcalloc(1+n, sizeof(int));
      /* allocate arrays to save parent subproblem components */
      tree->pred_m = m;
      tree->pred_n = n;
      tree->pred_max = m+n;
      tree->pred_type = xcalloc(1+m+n, sizeof(int));
      tree->pred_lb = xcalloc(1+m+n, sizeof(double));
      tree->pred_ub = xcalloc(1+m+n, sizeof(double));
      tree->pred_stat = xcalloc(1+m+n, sizeof(int));
      /* initialize control parameters with default values */
      tree->msg_lev = GLP_MSG_ALL;
      tree->br_tech = GLP_BR_DTH;
      tree->bt_tech = GLP_BT_BPH;
      tree->tol_int = 1e-5;
      tree->tol_obj = 1e-7;
      tree->tm_lim = INT_MAX;
      tree->out_frq = 5000;
      tree->out_dly = 10000;
      tree->tm_beg = xtime();
      tree->tm_lag = ulset(0, 0);
      /* initialize advanced solver interface */
      tree->cb_func = NULL;
      tree->cb_info = NULL;
      tree->reason = 0;
      tree->terminate = 0;
      return tree;
}

/***********************************************************************
*  NAME
*
*  ios_revive_node - revive specified subproblem
*
*  SYNOPSIS
*
*  #include "glpios.h"
*  void ios_revive_node(glp_tree *tree, int p);
*
*  DESCRIPTION
*
*  The routine ios_revive_node revives the specified subproblem, whose
*  reference number is p, and thereby makes it the current subproblem.
*  Note that the specified subproblem must be active. Besides, if the
*  current subproblem already exists, it must be frozen before reviving
*  another subproblem. */

void ios_revive_node(glp_tree *tree, int p)
{     glp_prob *mip = tree->mip;
      IOSNPD *node, *root;
      IOSBND *bnd;
      IOSTAT *stat;
      int i, j;
      /* obtain pointer to the specified subproblem */
      xassert(1 <= p && p <= tree->nslots);
      node = tree->slot[p].node;
      xassert(node != NULL);
      /* the specified subproblem must be active */
      xassert(node->count == 0);
      /* the current subproblem must not exist */
      xassert(tree->curr == NULL);
      /* the specified subproblem becomes current */
      tree->curr = node;
      /* obtain pointer to the root subproblem */
      root = tree->slot[1].node;
      xassert(root != NULL);
      /* build the path from the root to the current node */
      node->temp = NULL;
      for (node = node; node != NULL; node = node->up)
      {  if (node->up == NULL)
            xassert(node == root);
         else
            node->up->temp = node;
      }
      /* remove all additional rows (cuts) on all levels */
      if (mip->m != tree->orig_m)
      {  int nrs, *num;
         nrs = mip->m - tree->orig_m;
         xassert(nrs > 0);
         num = xcalloc(1+nrs, sizeof(int));
         for (i = 1; i <= nrs; i++)
            num[i] = tree->orig_m + i;
         glp_del_rows(mip, nrs, num);
         xfree(num);
      }
      xassert(mip->m == tree->orig_m);
      xassert(mip->n == tree->orig_n);
      /* assign original attributes to rows and columns */
      for (i = 1; i <= mip->m; i++)
      {  glp_set_row_bnds(mip, i, tree->orig_type[i],
            tree->orig_lb[i], tree->orig_ub[i]);
         glp_set_row_stat(mip, i, tree->orig_stat[i]);
      }
      for (j = 1; j <= mip->n; j++)
      {  glp_set_col_bnds(mip, j, tree->orig_type[mip->m+j],
            tree->orig_lb[mip->m+j], tree->orig_ub[mip->m+j]);
         glp_set_col_stat(mip, j, tree->orig_stat[mip->m+j]);
      }
      /* walk from the root to the current node and apply patch lists
         to restore the current subproblem */
      for (node = root; node != NULL; node = node->temp)
      {  int m = mip->m;
         int n = mip->n;
         /* if the current node has been reached, save attributes of
            rows and columns which now correspond to the parent of the
            specified subproblem */
         if (node->temp == NULL)
         {  tree->pred_m = m;
            tree->pred_n = n;
            /* reallocate arrays, if necessary */
            if (tree->pred_max < m + n)
            {  int new_size = m + n + 100;
               xfree(tree->pred_type);
               xfree(tree->pred_lb);
               xfree(tree->pred_ub);
               xfree(tree->pred_stat);
               tree->pred_max = new_size;
               tree->pred_type = xcalloc(1+new_size, sizeof(int));
               tree->pred_lb = xcalloc(1+new_size, sizeof(double));
               tree->pred_ub = xcalloc(1+new_size, sizeof(double));
               tree->pred_stat = xcalloc(1+new_size, sizeof(int));
            }
            for (i = 1; i <= m; i++)
            {  GLPROW *row = mip->row[i];
               tree->pred_type[i] = row->type;
               tree->pred_lb[i] = row->lb;
               tree->pred_ub[i] = row->ub;
               tree->pred_stat[i] = row->stat;
            }
            for (j = 1; j <= n; j++)
            {  GLPCOL *col = mip->col[j];
               tree->pred_type[mip->m+j] = col->type;
               tree->pred_lb[mip->m+j] = col->lb;
               tree->pred_ub[mip->m+j] = col->ub;
               tree->pred_stat[mip->m+j] = col->stat;
            }
         }
         /* change bounds of rows and columns */
         for (bnd = node->b_ptr; bnd != NULL; bnd = bnd->next)
         {  if (bnd->k <= m)
               glp_set_row_bnds(mip, bnd->k, bnd->type, bnd->lb,
                  bnd->ub);
            else
               glp_set_col_bnds(mip, bnd->k - m, bnd->type, bnd->lb,
                  bnd->ub);
         }
         /* change statuses of rows and columns */
         for (stat = node->s_ptr; stat != NULL; stat = stat->next)
         {  if (stat->k <= m)
               glp_set_row_stat(mip, stat->k, stat->stat);
            else
               glp_set_col_stat(mip, stat->k - m, stat->stat);
         }
         /* add new rows (cuts) */
         if (node->r_ptr != NULL)
         {  IOSROW *row;
            IOSAIJ *aij;
            int len, *ind;
            double *val;
            ind = xcalloc(1+n, sizeof(int));
            val = xcalloc(1+n, sizeof(double));
            for (row = node->r_ptr; row != NULL; row = row->next)
            {  i = glp_add_rows(mip, 1);
               if (row->name != NULL)
               {  char *name = mip->str_buf;
                  scs_get(name, row->name);
                  glp_set_row_name(mip, i, name);
               }
               glp_set_row_bnds(mip, i, row->type, row->lb, row->ub);
               len = 0;
               for (aij = row->ptr; aij != NULL; aij = aij->next)
                  len++, ind[len] = aij->j, val[len] = aij->val;
               glp_set_mat_row(mip, i, len, ind, val);
               glp_set_rii(mip, i, row->rii);
               glp_set_row_stat(mip, i, row->stat);
            }
            xfree(ind);
            xfree(val);
         }
      }
      /* the specified subproblem has been revived */
      node = tree->curr;
      /* delete its bound patch list */
      while (node->b_ptr != NULL)
      {  bnd = node->b_ptr;
         node->b_ptr = bnd->next;
         dmp_free_atom(tree->pool, bnd, sizeof(IOSBND));
      }
      /* delete its status patch list */
      while (node->s_ptr != NULL)
      {  stat = node->s_ptr;
         node->s_ptr = stat->next;
         dmp_free_atom(tree->pool, stat, sizeof(IOSTAT));
      }
      /* the specified subproblem cannot have additional rows */
      xassert(node->r_ptr == NULL);
      return;
}

/***********************************************************************
*  NAME
*
*  ios_freeze_node - freeze current subproblem
*
*  SYNOPSIS
*
*  #include "glpios.h"
*  void ios_freeze_node(glp_tree *tree);
*
*  DESCRIPTION
*
*  The routine ios_freeze_node freezes the current subproblem. */

void ios_freeze_node(glp_tree *tree)
{     glp_prob *mip = tree->mip;
      int pred_m = tree->pred_m;
      int pred_n = tree->pred_n;
      IOSNPD *node;
      int k;
      /* obtain pointer to the current subproblem */
      node = tree->curr;
      xassert(node != NULL);
      xassert(pred_m <= mip->m);
      xassert(pred_n == mip->n);
      /* build patch lists for rows and columns, which exist in the
         parent subproblem */
      xassert(node->b_ptr == NULL);
      xassert(node->s_ptr == NULL);
      for (k = 1; k <= pred_m + pred_n; k++)
      {  int pred_type, pred_stat, type, stat;
         double pred_lb, pred_ub, lb, ub;
         /* determine attributes in the parent subproblem */
         pred_type = tree->pred_type[k];
         pred_lb = tree->pred_lb[k];
         pred_ub = tree->pred_ub[k];
         pred_stat = tree->pred_stat[k];
         /* determine attributes in the current subproblem */
         if (k <= pred_m)
         {  GLPROW *row = mip->row[k];
            type = row->type;
            lb = row->lb;
            ub = row->ub;
            stat = row->stat;
         }
         else
         {  GLPCOL *col = mip->col[k - pred_m];
            type = col->type;
            lb = col->lb;
            ub = col->ub;
            stat = col->stat;
         }
         /* save type and bounds of row/column, if changed */
         if (!(pred_type == type && pred_lb == lb && pred_ub == ub))
         {  IOSBND *bnd;
            bnd = dmp_get_atom(tree->pool, sizeof(IOSBND));
            bnd->k = k;
            bnd->type = type;
            bnd->lb = lb;
            bnd->ub = ub;
            bnd->next = node->b_ptr;
            node->b_ptr = bnd;
         }
         /* save status of row/column, if changed */
         if (pred_stat != stat)
         {  IOSTAT *ssss;
            ssss = dmp_get_atom(tree->pool, sizeof(IOSTAT));
            ssss->k = k;
            ssss->stat = stat;
            ssss->next = node->s_ptr;
            node->s_ptr = ssss;
         }
      }
      /* save new rows, which have been added to the subproblem */
      xassert(node->r_ptr == NULL);
      if (pred_m < mip->m)
      {  int i, len, *ind;
         double *val;
         ind = xcalloc(1+mip->n, sizeof(int));
         val = xcalloc(1+mip->n, sizeof(double));
         for (i = mip->m; i > pred_m; i--)
         {  IOSROW *row;
            const char *name;
            row = dmp_get_atom(tree->pool, sizeof(IOSROW));
            name = glp_get_row_name(mip, i);
            if (name == NULL)
               row->name = NULL;
            else
            {  row->name = scs_new(tree->pool);
               scs_set(tree->pool, row->name, name);
            }
            row->type = mip->row[i]->type;
            row->lb = mip->row[i]->lb;
            row->ub = mip->row[i]->ub;
            row->ptr = NULL;
            len = glp_get_mat_row(mip, i, ind, val);
            for (k = 1; k <= len; k++)
            {  IOSAIJ *aij;
               aij = dmp_get_atom(tree->pool, sizeof(IOSAIJ));
               aij->j = ind[k];
               aij->val = val[k];
               aij->next = row->ptr;
               row->ptr = aij;
            }
            row->rii = mip->row[i]->rii;
            row->stat = mip->row[i]->stat;
            row->next = node->r_ptr;
            node->r_ptr = row;
         }
         xfree(ind);
         xfree(val);
      }
      /* the current subproblem has been frozen */
      tree->curr = NULL;
      return;
}

/***********************************************************************
*  NAME
*
*  ios_clone_node - clone specified subproblem
*
*  SYNOPSIS
*
*  #include "glpios.h"
*  void ios_clone_node(glp_tree *tree, int p, int nnn, int ref[]);
*
*  DESCRIPTION
*
*  The routine ios_clone_node clones the specified subproblem, whose
*  reference number is p, creating its nnn exact copies. Note that the
*  specified subproblem must be active and must be in the frozen state
*  (i.e. it must not be the current subproblem).
*
*  Each clone, an exact copy of the specified subproblem, becomes a new
*  active subproblem added to the end of the active list. After cloning
*  the specified subproblem becomes inactive.
*
*  The reference numbers of clone subproblems are stored to locations
*  ref[1], ..., ref[nnn]. */

void ios_clone_node(glp_tree *tree, int p, int nnn, int ref[])
{     IOSNPD *node, *orig;
      int k;
      /* obtain pointer to the subproblem to be cloned */
      xassert(1 <= p && p <= tree->nslots);
      node = tree->slot[p].node;
      xassert(node != NULL);
      /* the specified subproblem must be active */
      xassert(node->count == 0);
      /* and must be in the frozen state */
      xassert(tree->curr != node);
      /* remove the specified subproblem from the active list, because
         it becomes inactive */
      if (node->prev == NULL)
         tree->head = node->next;
      else
         node->prev->next = node->next;
      if (node->next == NULL)
         tree->tail = node->prev;
      else
         node->next->prev = node->prev;
      node->prev = node->next = NULL;
      tree->a_cnt--;
      /* set the child count of the specified subproblem, which is the
         number of clones to be created */
      xassert(nnn > 0);
      node->count = nnn;
      /* save pointer to the specified subproblem */
      orig = node;
      /* create clone subproblems */
      for (k = 1; k <= nnn; k++)
      {  /* if no free slots are available, increase the room */
         if (tree->avail == 0)
         {  int nslots = tree->nslots;
            IOSLOT *save = tree->slot;
            tree->nslots = nslots + nslots;
            xassert(tree->nslots > nslots);
            tree->slot = xcalloc(1+tree->nslots, sizeof(IOSLOT));
            memcpy(&tree->slot[1], &save[1], nslots * sizeof(IOSLOT));
            /* push more free slots into the stack */
            for (p = tree->nslots; p > nslots; p--)
            {  tree->slot[p].node = NULL;
               tree->slot[p].next = tree->avail;
               tree->avail = p;
            }
            xfree(save);
         }
         /* pull a free slot from the stack */
         p = tree->avail;
         tree->avail = tree->slot[p].next;
         xassert(tree->slot[p].node == NULL);
         tree->slot[p].next = 0;
         /* create descriptor for new subproblem */
         node = dmp_get_atom(tree->pool, sizeof(IOSNPD));
         tree->slot[p].node = node;
         node->p = p;
         node->up = orig;
         node->level = orig->level + 1;
         node->count = 0;
         node->r_ptr = NULL;
         node->b_ptr = NULL;
         node->s_ptr = NULL;
         node->bound = orig->bound;
         node->ii_cnt = 0;
         node->ii_sum = 0.0;
         node->temp = NULL;
         node->prev = tree->tail;
         node->next = NULL;
         /* add the new subproblem to the end of the active list */
         if (tree->head == NULL)
            tree->head = node;
         else
            tree->tail->next = node;
         tree->tail = node;
         tree->a_cnt++;
         tree->n_cnt++;
         tree->t_cnt++;
         /* store the clone subproblem reference number */
         ref[k] = p;
      }
      return;
}

/***********************************************************************
*  NAME
*
*  ios_delete_node - delete specified subproblem
*
*  SYNOPSIS
*
*  #include "glpios.h"
*  void ios_delete_node(glp_tree *tree, int p);
*
*  DESCRIPTION
*
*  The routine ios_delete_node deletes the specified subproblem, whose
*  reference number is p. The subproblem must be active and must be in
*  the frozen state (i.e. it must not be the current subproblem).
*
*  Note that deletion is performed recursively, i.e. if a subproblem to
*  be deleted is the only child of its parent, the parent subproblem is
*  also deleted, etc. */

void ios_delete_node(glp_tree *tree, int p)
{     IOSNPD *node, *temp;
      IOSBND *bnds;
      IOSTAT *stat;
      /* obtain pointer to the subproblem to be deleted */
      xassert(1 <= p && p <= tree->nslots);
      node = tree->slot[p].node;
      xassert(node != NULL);
      /* the specified subproblem must be active */
      xassert(node->count == 0);
      /* and must be in the frozen state */
      xassert(tree->curr != node);
      /* remove the specified subproblem from the active list, because
         it is gone from the tree */
      if (node->prev == NULL)
         tree->head = node->next;
      else
         node->prev->next = node->next;
      if (node->next == NULL)
         tree->tail = node->prev;
      else
         node->next->prev = node->prev;
      node->prev = node->next = NULL;
      tree->a_cnt--;
loop: /* recursive deletion starts here */
      /* delete the bounds patch list */
      while (node->b_ptr != NULL)
      {  bnds = node->b_ptr;
         node->b_ptr = bnds->next;
         dmp_free_atom(tree->pool, bnds, sizeof(IOSBND));
      }
      /* delete the statuses patch list */
      while (node->s_ptr != NULL)
      {  stat = node->s_ptr;
         node->s_ptr = stat->next;
         dmp_free_atom(tree->pool, stat, sizeof(IOSTAT));
      }
      /* delete the row addition list */
      while (node->r_ptr != NULL)
      {  IOSROW *row;
         row = node->r_ptr;
         if (row->name != NULL)
            scs_drop(tree->pool, row->name);
         while (row->ptr != NULL)
         {  IOSAIJ *aij;
            aij = row->ptr;
            row->ptr = aij->next;
            dmp_free_atom(tree->pool, aij, sizeof(IOSAIJ));
         }
         node->r_ptr = row->next;
         dmp_free_atom(tree->pool, row, sizeof(IOSROW));
      }
      /* free the corresponding node slot */
      p = node->p;
      xassert(tree->slot[p].node == node);
      tree->slot[p].node = NULL;
      tree->slot[p].next = tree->avail;
      tree->avail = p;
      /* save pointer to the parent subproblem */
      temp = node->up;
      /* delete the subproblem descriptor */
      dmp_free_atom(tree->pool, node, sizeof(IOSNPD));
      tree->n_cnt--;
      /* take pointer to the parent subproblem */
      node = temp;
      if (node != NULL)
      {  /* the parent subproblem exists; decrease the number of its
            child subproblems */
         xassert(node->count > 0);
         node->count--;
         /* if now the parent subproblem has no childs, it also must be
            deleted */
         if (node->count == 0) goto loop;
      }
      return;
}

/***********************************************************************
*  NAME
*
*  ios_delete_tree - delete branch-and-bound tree
*
*  SYNOPSIS
*
*  #include "glpios.h"
*  void ios_delete_tree(glp_tree *tree);
*
*  DESCRIPTION
*
*  The routine ios_delete_tree deletes the branch-and-bound tree, which
*  the parameter tree points to. That frees all the memory allocated to
*  the integer optimization suite. */

void ios_delete_tree(glp_tree *tree)
{     glp_prob *mip = tree->mip;
      int i, j;
      /* remove all additional rows (cuts) on all levels */
      if (mip->m != tree->orig_m)
      {  int nrs, *num;
         nrs = mip->m - tree->orig_m;
         xassert(nrs > 0);
         num = xcalloc(1+nrs, sizeof(int));
         for (i = 1; i <= nrs; i++)
            num[i] = tree->orig_m + i;
         glp_del_rows(mip, nrs, num);
         xfree(num);
      }
      xassert(mip->m == tree->orig_m);
      xassert(mip->n == tree->orig_n);
      /* restore original problem components */
      for (i = 1; i <= mip->m; i++)
      {  glp_set_row_bnds(mip, i, tree->orig_type[i],
            tree->orig_lb[i], tree->orig_ub[i]);
         glp_set_row_stat(mip, i, tree->orig_stat[i]);
         mip->row[i]->prim = tree->orig_prim[i];
         mip->row[i]->dual = tree->orig_dual[i];
      }
      for (j = 1; j <= mip->n; j++)
      {  glp_set_col_bnds(mip, j, tree->orig_type[mip->m+j],
            tree->orig_lb[mip->m+j], tree->orig_ub[mip->m+j]);
         glp_set_col_stat(mip, j, tree->orig_stat[mip->m+j]);
         mip->col[j]->prim = tree->orig_prim[mip->m+j];
         mip->col[j]->dual = tree->orig_dual[mip->m+j];
      }
      mip->pbs_stat = mip->dbs_stat = GLP_FEAS;
      mip->obj_val = tree->orig_obj;
      /* delete the branch-and-bound tree */
      dmp_delete_pool(tree->pool);
      xfree(tree->orig_type);
      xfree(tree->orig_lb);
      xfree(tree->orig_ub);
      xfree(tree->orig_stat);
      xfree(tree->orig_prim);
      xfree(tree->orig_dual);
      xfree(tree->slot);
      xfree(tree->non_int);
      xfree(tree->pred_type);
      xfree(tree->pred_lb);
      xfree(tree->pred_ub);
      xfree(tree->pred_stat);
      xfree(tree);
      return;
}

/***********************************************************************
*  NAME
*
*  ios_best_node - find active node with best local bound
*
*  SYNOPSIS
*
*  #include "glpios.h"
*  int ios_best_node(glp_tree *tree);
*
*  DESCRIPTION
*
*  The routine ios_best_node finds an active node whose local bound is
*  best among other active nodes.
*
*  It is understood that the integer optimal solution of the original
*  mip problem cannot be better than the best bound, so the best bound
*  is an lower (minimization) or upper (maximization) global bound for
*  the original problem.
*
*  RETURNS
*
*  The routine ios_best_node returns the subproblem reference number
*  for the best node. However, if the tree is empty, it returns zero. */

int ios_best_node(glp_tree *tree)
{     IOSNPD *node, *best = NULL;
      switch (tree->mip->dir)
      {  case GLP_MIN:
            /* minimization */
            for (node = tree->head; node != NULL; node = node->next)
               if (best == NULL || best->bound > node->bound)
                  best = node;
            break;
         case GLP_MAX:
            /* maximization */
            for (node = tree->head; node != NULL; node = node->next)
               if (best == NULL || best->bound < node->bound)
                  best = node;
            break;
         default:
            xassert(tree != tree);
      }
      return best == NULL ? 0 : best->p;
}

/***********************************************************************
*  NAME
*
*  ios_relative_gap - compute relative mip gap
*
*  SYNOPSIS
*
*  #include "glpios.h"
*  double ios_relative_gap(glp_tree *tree);
*
*  DESCRIPTION
*
*  The routine ios_relative_gap computes the relative mip gap using the
*  formula:
*
*     gap = |best_mip - best_bnd| / (|best_mip| + DBL_EPSILON),
*
*  where best_mip is the best integer feasible solution found so far,
*  best_bnd is the best (global) bound. If no integer feasible solution
*  has been found yet, rel_gap is set to DBL_MAX.
*
*  RETURNS
*
*  The routine ios_relative_gap returns the relative mip gap. */

double ios_relative_gap(glp_tree *tree)
{     glp_prob *mip = tree->mip;
      int p;
      double best_mip, best_bnd, gap;
      if (mip->mip_stat == GLP_FEAS)
      {  best_mip = mip->mip_obj;
         p = ios_best_node(tree);
         if (p == 0)
         {  /* the tree is empty */
            gap = 0.0;
         }
         else
         {  best_bnd = tree->slot[p].node->bound;
            gap = fabs(best_mip - best_bnd) / (fabs(best_mip) +
               DBL_EPSILON);
         }
      }
      else
      {  /* no integer feasible solution has been found yet */
         gap = DBL_MAX;
      }
      return gap;
}

/***********************************************************************
*  NAME
*
*  ios_solve_node - solve LP relaxation of current subproblem
*
*  SYNOPSIS
*
*  #include "glpios.h"
*  int ios_solve_node(glp_tree *tree);
*
*  DESCRIPTION
*
*  The routine ios_solve_node solves or re-optimizes LP relaxation of
*  the current subproblem using the simplex method.
*
*  RETURNS
*
*  The routine returns the code which is reported by glp_simplex. */

int ios_solve_node(glp_tree *tree)
{     glp_prob *mip = tree->mip;
      glp_smcp parm;
      int ret;
      /* the current subproblem must exist */
      xassert(tree->curr != NULL);
      /* set some control parameters */
      glp_init_smcp(&parm);
      switch (tree->msg_lev)
      {  case GLP_MSG_OFF:
            parm.msg_lev = GLP_MSG_OFF; break;
         case GLP_MSG_ERR:
            parm.msg_lev = GLP_MSG_ERR; break;
         case GLP_MSG_ON:
         case GLP_MSG_ALL:
            parm.msg_lev = GLP_MSG_ON; break;
         case GLP_MSG_DBG:
            parm.msg_lev = GLP_MSG_ALL; break;
         default:
            xassert(tree != tree);
      }
      parm.meth = GLP_DUALP;
      if (tree->msg_lev < GLP_MSG_DBG)
         parm.out_dly = tree->out_dly;
      else
         parm.out_dly = 0;
      /* if the incumbent objective value is already known, use it to
         prematurely terminate the dual simplex search */
      if (mip->mip_stat == GLP_FEAS)
      {  switch (tree->mip->dir)
         {  case GLP_MIN:
               parm.obj_ul = mip->mip_obj;
               break;
            case GLP_MAX:
               parm.obj_ll = mip->mip_obj;
               break;
            default:
               xassert(mip != mip);
         }
      }
      /* try to solve/re-optimize the LP relaxation */
      ret = glp_simplex(mip, &parm);
      return ret;
}

/* eof */
